/*
  LogicHub Dock Firmware v1

  Firmware for LogicHub Dock Rev.1

  Board : METE HOCA Akana R1
  Dock  : LogicHub Dock Rev.1
  Core  : earlephilhower / arduino-pico (https://github.com/earlephilhower/arduino-pico)
  Required Libraries: U8G2 (https://github.com/olikraus/u8g2)
  Settings: Select METE HOCA Akana R1 as the board.
            Set the I2C selector to the I2C0 position.

  Source: METE HOCA
  
  METE HOCA, 31 July 2024
  IG : https://www.instagram.com/metehoca/
  YT : https://www.youtube.com/MeteHoca
  WEB: https://www.metehoca.com/
*/

#include <Wire.h>
#include <U8g2lib.h>
#include "images.h"

U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, U8X8_PIN_NONE);

const char* nameAND = "AND";   // 0
const char* nameOR = "OR";     // 1
const char* nameNAND = "NAND"; // 2
const char* nameNOR = "NOR";   // 3
const char* nameXOR = "XOR";   // 4
const char* nameXNOR = "XNOR"; // 5
const char* nameNOT = "NOT";   // 6
const char* nameBUF = "BUF";   // 7

int A_IN1 = 0; int A_IN2 = 1; int A_OUT = 2;
int B_IN1 = 3; int B_IN2 = 6; int B_OUT = 7;
int C_IN1 = 8; int C_IN2 = 9; int C_OUT = 10;
int D_IN1 = 11; int D_IN2 = 12; int D_OUT = 13;
int E_IN1 = 16; int E_IN2 = 17; int E_OUT = 18;
int F_IN1 = 19; int F_IN2 = 24; int F_OUT = 26;
int G_IN1 = 27; int G_IN2 = 28; int G_OUT = 29;

int menu = 0;
bool menuOpen = false;
int gateStatus[7] = {0, 1, 2, 3, 4, 5, 6};
int gateTempStatus[7] = {0, 1, 2, 3, 4, 5, 6};

void setup() {
  u8g2.begin();
  u8g2.setFont(u8g2_font_squeezed_r7_tr);
  u8g2.setFontMode(1);
  u8g2.setDrawColor(1);
  u8g2.setFontPosTop();
  u8g2.setContrast(255);
  
  pinMode(BTN_UP, INPUT);
  pinMode(BTN_DOWN, INPUT);
  pinMode(BTN_LEFT, INPUT);
  pinMode(BTN_RIGHT, INPUT);
  pinMode(BTN_BACK, INPUT);
  pinMode(BTN_ENTER, INPUT);

  pinMode(A_IN1, INPUT_PULLDOWN); pinMode(A_IN2, INPUT_PULLDOWN); pinMode(A_OUT, OUTPUT_12MA);
  pinMode(B_IN1, INPUT_PULLDOWN); pinMode(B_IN2, INPUT_PULLDOWN); pinMode(B_OUT, OUTPUT_12MA);
  pinMode(C_IN1, INPUT_PULLDOWN); pinMode(C_IN2, INPUT_PULLDOWN); pinMode(C_OUT, OUTPUT_12MA);
  pinMode(D_IN1, INPUT_PULLDOWN); pinMode(D_IN2, INPUT_PULLDOWN); pinMode(D_OUT, OUTPUT_12MA);
  pinMode(E_IN1, INPUT_PULLDOWN); pinMode(E_IN2, INPUT_PULLDOWN); pinMode(E_OUT, OUTPUT_12MA);
  pinMode(F_IN1, INPUT_PULLDOWN); pinMode(F_IN2, INPUT_PULLDOWN); pinMode(F_OUT, OUTPUT_12MA);
  pinMode(G_IN1, INPUT_PULLDOWN); pinMode(G_IN2, INPUT_PULLDOWN); pinMode(G_OUT, OUTPUT_12MA);

  u8g2.clearBuffer();
  u8g2.drawXBMP(0, 14, 128, 36, AKANA);
  u8g2.sendBuffer();
  delay(1000);

  attachInterrupt(digitalPinToInterrupt(BTN_LEFT), menuLeft, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_RIGHT), menuRight, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_UP), menuUp, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_DOWN), menuDown, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_ENTER), menuEnter, RISING);
  attachInterrupt(digitalPinToInterrupt(BTN_BACK), menuBack, RISING);
}

void loop() {
  noInterrupts();
  showMenu(); u8g2.sendBuffer();
  interrupts();
  delay(50);

  if (gateStatus[0] == 0) digitalWriteFast(A_OUT, (digitalRead(A_IN1) & digitalRead(A_IN2)));  // AND
  if (gateStatus[0] == 1) digitalWriteFast(A_OUT, (digitalRead(A_IN1) | digitalRead(A_IN2)));  // OR
  if (gateStatus[0] == 2) digitalWriteFast(A_OUT, !(digitalRead(A_IN1) & digitalRead(A_IN2))); // NAND
  if (gateStatus[0] == 3) digitalWriteFast(A_OUT, !(digitalRead(A_IN1) | digitalRead(A_IN2))); // NOR
  if (gateStatus[0] == 4) digitalWriteFast(A_OUT, (digitalRead(A_IN1) ^ digitalRead(A_IN2)));  // XOR
  if (gateStatus[0] == 5) digitalWriteFast(A_OUT, !(digitalRead(A_IN1) ^ digitalRead(A_IN2))); // XNOR
  if (gateStatus[0] == 6) digitalWriteFast(A_OUT, !(digitalRead(A_IN1)));                      // NOT
  if (gateStatus[0] == 7) digitalWriteFast(A_OUT, (digitalRead(A_IN1)));                       // BUFFER

  if (gateStatus[1] == 0) digitalWriteFast(B_OUT, (digitalRead(B_IN1) & digitalRead(B_IN2)));  // AND
  if (gateStatus[1] == 1) digitalWriteFast(B_OUT, (digitalRead(B_IN1) | digitalRead(B_IN2)));  // OR
  if (gateStatus[1] == 2) digitalWriteFast(B_OUT, !(digitalRead(B_IN1) & digitalRead(B_IN2))); // NAND
  if (gateStatus[1] == 3) digitalWriteFast(B_OUT, !(digitalRead(B_IN1) | digitalRead(B_IN2))); // NOR
  if (gateStatus[1] == 4) digitalWriteFast(B_OUT, (digitalRead(B_IN1) ^ digitalRead(B_IN2)));  // XOR
  if (gateStatus[1] == 5) digitalWriteFast(B_OUT, !(digitalRead(B_IN1) ^ digitalRead(B_IN2))); // XNOR
  if (gateStatus[1] == 6) digitalWriteFast(B_OUT, !(digitalRead(B_IN1)));                      // NOT
  if (gateStatus[1] == 7) digitalWriteFast(B_OUT, (digitalRead(B_IN1)));                       // BUFFER

  if (gateStatus[2] == 0) digitalWriteFast(C_OUT, (digitalRead(C_IN1) & digitalRead(C_IN2)));  // AND
  if (gateStatus[2] == 1) digitalWriteFast(C_OUT, (digitalRead(C_IN1) | digitalRead(C_IN2)));  // OR
  if (gateStatus[2] == 2) digitalWriteFast(C_OUT, !(digitalRead(C_IN1) & digitalRead(C_IN2))); // NAND
  if (gateStatus[2] == 3) digitalWriteFast(C_OUT, !(digitalRead(C_IN1) | digitalRead(C_IN2))); // NOR
  if (gateStatus[2] == 4) digitalWriteFast(C_OUT, (digitalRead(C_IN1) ^ digitalRead(C_IN2)));  // XOR
  if (gateStatus[2] == 5) digitalWriteFast(C_OUT, !(digitalRead(C_IN1) ^ digitalRead(C_IN2))); // XNOR
  if (gateStatus[2] == 6) digitalWriteFast(C_OUT, !(digitalRead(C_IN1)));                      // NOT
  if (gateStatus[2] == 7) digitalWriteFast(C_OUT, (digitalRead(C_IN1)));                       // BUFFER

  if (gateStatus[3] == 0) digitalWriteFast(D_OUT, (digitalRead(D_IN1) & digitalRead(D_IN2)));  // AND
  if (gateStatus[3] == 1) digitalWriteFast(D_OUT, (digitalRead(D_IN1) | digitalRead(D_IN2)));  // OR
  if (gateStatus[3] == 2) digitalWriteFast(D_OUT, !(digitalRead(D_IN1) & digitalRead(D_IN2))); // NAND
  if (gateStatus[3] == 3) digitalWriteFast(D_OUT, !(digitalRead(D_IN1) | digitalRead(D_IN2))); // NOR
  if (gateStatus[3] == 4) digitalWriteFast(D_OUT, (digitalRead(D_IN1) ^ digitalRead(D_IN2)));  // XOR
  if (gateStatus[3] == 5) digitalWriteFast(D_OUT, !(digitalRead(D_IN1) ^ digitalRead(D_IN2))); // XNOR
  if (gateStatus[3] == 6) digitalWriteFast(D_OUT, !(digitalRead(D_IN1)));                      // NOT
  if (gateStatus[3] == 7) digitalWriteFast(D_OUT, (digitalRead(D_IN1)));                       // BUFFER

  if (gateStatus[4] == 0) digitalWriteFast(E_OUT, (digitalRead(E_IN1) & digitalRead(E_IN2)));  // AND
  if (gateStatus[4] == 1) digitalWriteFast(E_OUT, (digitalRead(E_IN1) | digitalRead(E_IN2)));  // OR
  if (gateStatus[4] == 2) digitalWriteFast(E_OUT, !(digitalRead(E_IN1) & digitalRead(E_IN2))); // NAND
  if (gateStatus[4] == 3) digitalWriteFast(E_OUT, !(digitalRead(E_IN1) | digitalRead(E_IN2))); // NOR
  if (gateStatus[4] == 4) digitalWriteFast(E_OUT, (digitalRead(E_IN1) ^ digitalRead(E_IN2)));  // XOR
  if (gateStatus[4] == 5) digitalWriteFast(E_OUT, !(digitalRead(E_IN1) ^ digitalRead(E_IN2))); // XNOR
  if (gateStatus[4] == 6) digitalWriteFast(E_OUT, !(digitalRead(E_IN1)));                      // NOT
  if (gateStatus[4] == 7) digitalWriteFast(E_OUT, (digitalRead(E_IN1)));                       // BUFFER

  if (gateStatus[5] == 0) digitalWriteFast(F_OUT, (digitalRead(F_IN1) & digitalRead(F_IN2)));  // AND
  if (gateStatus[5] == 1) digitalWriteFast(F_OUT, (digitalRead(F_IN1) | digitalRead(F_IN2)));  // OR
  if (gateStatus[5] == 2) digitalWriteFast(F_OUT, !(digitalRead(F_IN1) & digitalRead(F_IN2))); // NAND
  if (gateStatus[5] == 3) digitalWriteFast(F_OUT, !(digitalRead(F_IN1) | digitalRead(F_IN2))); // NOR
  if (gateStatus[5] == 4) digitalWriteFast(F_OUT, (digitalRead(F_IN1) ^ digitalRead(F_IN2)));  // XOR
  if (gateStatus[5] == 5) digitalWriteFast(F_OUT, !(digitalRead(F_IN1) ^ digitalRead(F_IN2))); // XNOR
  if (gateStatus[5] == 6) digitalWriteFast(F_OUT, !(digitalRead(F_IN1)));                      // NOT
  if (gateStatus[5] == 7) digitalWriteFast(F_OUT, (digitalRead(F_IN1)));                       // BUFFER

  if (gateStatus[6] == 0) digitalWriteFast(G_OUT, (digitalRead(G_IN1) & digitalRead(G_IN2)));  // AND
  if (gateStatus[6] == 1) digitalWriteFast(G_OUT, (digitalRead(G_IN1) | digitalRead(G_IN2)));  // OR
  if (gateStatus[6] == 2) digitalWriteFast(G_OUT, !(digitalRead(G_IN1) & digitalRead(G_IN2))); // NAND
  if (gateStatus[6] == 3) digitalWriteFast(G_OUT, !(digitalRead(G_IN1) | digitalRead(G_IN2))); // NOR
  if (gateStatus[6] == 4) digitalWriteFast(G_OUT, (digitalRead(G_IN1) ^ digitalRead(G_IN2)));  // XOR
  if (gateStatus[6] == 5) digitalWriteFast(G_OUT, !(digitalRead(G_IN1) ^ digitalRead(G_IN2))); // XNOR
  if (gateStatus[6] == 6) digitalWriteFast(G_OUT, !(digitalRead(G_IN1)));                      // NOT
  if (gateStatus[6] == 7) digitalWriteFast(G_OUT, (digitalRead(G_IN1)));                       // BUFFER
}

void menuEnter() {
  if (menuOpen == false) {
    menuOpen = true;
    gateTempStatus[menu] = gateStatus[menu];
  } else {
    menuOpen = false;
    gateStatus[menu] = gateTempStatus[menu];
  }
}

void menuBack() {
  if (menuOpen == true) {
    menuOpen = false;
    gateTempStatus[menu] = gateStatus[menu];
  }
}

void menuLeft() {
  if (menuOpen == false) {
    if (menu == 0) {menu = 6; return;}
    if (menu == 1) {menu = 0; return;}
    if (menu == 2) {menu = 1; return;}
    if (menu == 3) {menu = 2; return;}
    if (menu == 4) {menu = 3; return;}
    if (menu == 5) {menu = 4; return;}
    if (menu == 6) {menu = 5; return;}
   } else {
    if (gateTempStatus[menu] == 0) gateTempStatus[menu] = 7; else gateTempStatus[menu]--;
  }
}

void menuRight() {
  if (menuOpen == false) {
    if (menu == 0) {menu = 1; return;}
    if (menu == 1) {menu = 2; return;}
    if (menu == 2) {menu = 3; return;}
    if (menu == 3) {menu = 4; return;}
    if (menu == 4) {menu = 5; return;}
    if (menu == 5) {menu = 6; return;}
    if (menu == 6) {menu = 0; return;}
  } else {
    if (gateTempStatus[menu] == 7) gateTempStatus[menu] = 0; else gateTempStatus[menu]++;
  }
}

void menuUp() {
  if (menuOpen == false) {
    if (menu == 0) {menu = 4; return;}
    if (menu == 1) {menu = 5; return;}
    if (menu == 2) {menu = 6; return;}
    if (menu == 3) {menu = 6; return;}
    if (menu == 4) {menu = 0; return;}
    if (menu == 5) {menu = 1; return;}
    if (menu == 6) {menu = 2; return;}
  }
}

void menuDown() {
  if (menuOpen == false) {
    if (menu == 0) {menu = 4; return;}
    if (menu == 1) {menu = 5; return;}
    if (menu == 2) {menu = 6; return;}
    if (menu == 3) {menu = 6; return;}
    if (menu == 4) {menu = 0; return;}
    if (menu == 5) {menu = 1; return;}
    if (menu == 6) {menu = 2; return;}
  }
}

void showMenu() {
  u8g2.clearBuffer();

  if (menuOpen == false) {
    if (gateStatus[0] == 0) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameAND))) / 2)), 20, nameAND); u8g2.drawXBMP(8, 3, 16, 16, gateAND);}
    if (gateStatus[0] == 1) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameOR))) / 2)), 20, nameOR); u8g2.drawXBMP(8, 3, 16, 16, gateOR);}
    if (gateStatus[0] == 2) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 20, nameNAND); u8g2.drawXBMP(8, 3, 16, 16, gateNAND);}
    if (gateStatus[0] == 3) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 20, nameNOR); u8g2.drawXBMP(8, 3, 16, 16, gateNOR);}
    if (gateStatus[0] == 4) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 20, nameXOR); u8g2.drawXBMP(8, 3, 16, 16, gateXOR);}
    if (gateStatus[0] == 5) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 20, nameXNOR); u8g2.drawXBMP(8, 3, 16, 16, gateXNOR);}
    if (gateStatus[0] == 6) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 20, nameNOT); u8g2.drawXBMP(8, 3, 16, 16, gateNOT);}
    if (gateStatus[0] == 7) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 20, nameBUF); u8g2.drawXBMP(8, 3, 16, 16, gateBUF);}

    if (gateStatus[1] == 0) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameAND))) / 2)), 20, nameAND); u8g2.drawXBMP(40, 3, 16, 16, gateAND);}
    if (gateStatus[1] == 1) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameOR))) / 2)), 20, nameOR); u8g2.drawXBMP(40, 3, 16, 16, gateOR);}
    if (gateStatus[1] == 2) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 20, nameNAND); u8g2.drawXBMP(40, 3, 16, 16, gateNAND);}
    if (gateStatus[1] == 3) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 20, nameNOR); u8g2.drawXBMP(40, 3, 16, 16, gateNOR);}
    if (gateStatus[1] == 4) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 20, nameXOR); u8g2.drawXBMP(40, 3, 16, 16, gateXOR);}
    if (gateStatus[1] == 5) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 20, nameXNOR); u8g2.drawXBMP(40, 3, 16, 16, gateXNOR);}
    if (gateStatus[1] == 6) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 20, nameNOT); u8g2.drawXBMP(40, 3, 16, 16, gateNOT);}
    if (gateStatus[1] == 7) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 20, nameBUF); u8g2.drawXBMP(40, 3, 16, 16, gateBUF);}

    if (gateStatus[2] == 0) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameAND))) / 2)), 20, nameAND); u8g2.drawXBMP(72, 3, 16, 16, gateAND);}
    if (gateStatus[2] == 1) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameOR))) / 2)), 20, nameOR); u8g2.drawXBMP(72, 3, 16, 16, gateOR);}
    if (gateStatus[2] == 2) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 20, nameNAND); u8g2.drawXBMP(72, 3, 16, 16, gateNAND);}
    if (gateStatus[2] == 3) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 20, nameNOR); u8g2.drawXBMP(72, 3, 16, 16, gateNOR);}
    if (gateStatus[2] == 4) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 20, nameXOR); u8g2.drawXBMP(72, 3, 16, 16, gateXOR);}
    if (gateStatus[2] == 5) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 20, nameXNOR); u8g2.drawXBMP(72, 3, 16, 16, gateXNOR);}
    if (gateStatus[2] == 6) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 20, nameNOT); u8g2.drawXBMP(72, 3, 16, 16, gateNOT);}
    if (gateStatus[2] == 7) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 20, nameBUF); u8g2.drawXBMP(72, 3, 16, 16, gateBUF);}

    if (gateStatus[3] == 0) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameAND))) / 2)), 20, nameAND); u8g2.drawXBMP(104, 3, 16, 16, gateAND);}
    if (gateStatus[3] == 1) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameOR))) / 2)), 20, nameOR); u8g2.drawXBMP(104, 3, 16, 16, gateOR);}
    if (gateStatus[3] == 2) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 20, nameNAND); u8g2.drawXBMP(104, 3, 16, 16, gateNAND);}
    if (gateStatus[3] == 3) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 20, nameNOR); u8g2.drawXBMP(104, 3, 16, 16, gateNOR);}
    if (gateStatus[3] == 4) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 20, nameXOR); u8g2.drawXBMP(104, 3, 16, 16, gateXOR);}
    if (gateStatus[3] == 5) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 20, nameXNOR); u8g2.drawXBMP(104, 3, 16, 16, gateXNOR);}
    if (gateStatus[3] == 6) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 20, nameNOT); u8g2.drawXBMP(104, 3, 16, 16, gateNOT);}
    if (gateStatus[3] == 7) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 20, nameBUF); u8g2.drawXBMP(104, 3, 16, 16, gateBUF);}

    if (gateStatus[4] == 0) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameAND))) / 2)), 52, nameAND); u8g2.drawXBMP(8, 35, 16, 16, gateAND);}
    if (gateStatus[4] == 1) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameOR))) / 2)), 52, nameOR); u8g2.drawXBMP(8, 35, 16, 16, gateOR);}
    if (gateStatus[4] == 2) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 52, nameNAND); u8g2.drawXBMP(8, 35, 16, 16, gateNAND);}
    if (gateStatus[4] == 3) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 52, nameNOR); u8g2.drawXBMP(8, 35, 16, 16, gateNOR);}
    if (gateStatus[4] == 4) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 52, nameXOR); u8g2.drawXBMP(8, 35, 16, 16, gateXOR);}
    if (gateStatus[4] == 5) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 52, nameXNOR); u8g2.drawXBMP(8, 35, 16, 16, gateXNOR);}
    if (gateStatus[4] == 6) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 52, nameNOT); u8g2.drawXBMP(8, 35, 16, 16, gateNOT);}
    if (gateStatus[4] == 7) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 52, nameBUF); u8g2.drawXBMP(8, 35, 16, 16, gateBUF);}

    if (gateStatus[5] == 0) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameAND))) / 2)), 52, nameAND); u8g2.drawXBMP(40, 35, 16, 16, gateAND);}
    if (gateStatus[5] == 1) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameOR))) / 2)), 52, nameOR); u8g2.drawXBMP(40, 35, 16, 16, gateOR);}
    if (gateStatus[5] == 2) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 52, nameNAND); u8g2.drawXBMP(40, 35, 16, 16, gateNAND);}
    if (gateStatus[5] == 3) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 52, nameNOR); u8g2.drawXBMP(40, 35, 16, 16, gateNOR);}
    if (gateStatus[5] == 4) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 52, nameXOR); u8g2.drawXBMP(40, 35, 16, 16, gateXOR);}
    if (gateStatus[5] == 5) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 52, nameXNOR); u8g2.drawXBMP(40, 35, 16, 16, gateXNOR);}
    if (gateStatus[5] == 6) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 52, nameNOT); u8g2.drawXBMP(40, 35, 16, 16, gateNOT);}
    if (gateStatus[5] == 7) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 52, nameBUF); u8g2.drawXBMP(40, 35, 16, 16, gateBUF);}

    if (gateStatus[6] == 0) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameAND))) / 2)), 52, nameAND); u8g2.drawXBMP(72, 35, 16, 16, gateAND);}
    if (gateStatus[6] == 1) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameOR))) / 2)), 52, nameOR); u8g2.drawXBMP(72, 35, 16, 16, gateOR);}
    if (gateStatus[6] == 2) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 52, nameNAND); u8g2.drawXBMP(72, 35, 16, 16, gateNAND);}
    if (gateStatus[6] == 3) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 52, nameNOR); u8g2.drawXBMP(72, 35, 16, 16, gateNOR);}
    if (gateStatus[6] == 4) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 52, nameXOR); u8g2.drawXBMP(72, 35, 16, 16, gateXOR);}
    if (gateStatus[6] == 5) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 52, nameXNOR); u8g2.drawXBMP(72, 35, 16, 16, gateXNOR);}
    if (gateStatus[6] == 6) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 52, nameNOT); u8g2.drawXBMP(72, 35, 16, 16, gateNOT);}
    if (gateStatus[6] == 7) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 52, nameBUF); u8g2.drawXBMP(72, 35, 16, 16, gateBUF);}
  } else {
    if (gateTempStatus[0] == 0) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameAND))) / 2)), 20, nameAND); u8g2.drawXBMP(8, 3, 16, 16, gateAND);}
    if (gateTempStatus[0] == 1) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameOR))) / 2)), 20, nameOR); u8g2.drawXBMP(8, 3, 16, 16, gateOR);}
    if (gateTempStatus[0] == 2) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 20, nameNAND); u8g2.drawXBMP(8, 3, 16, 16, gateNAND);}
    if (gateTempStatus[0] == 3) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 20, nameNOR); u8g2.drawXBMP(8, 3, 16, 16, gateNOR);}
    if (gateTempStatus[0] == 4) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 20, nameXOR); u8g2.drawXBMP(8, 3, 16, 16, gateXOR);}
    if (gateTempStatus[0] == 5) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 20, nameXNOR); u8g2.drawXBMP(8, 3, 16, 16, gateXNOR);}
    if (gateTempStatus[0] == 6) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 20, nameNOT); u8g2.drawXBMP(8, 3, 16, 16, gateNOT);}
    if (gateTempStatus[0] == 7) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 20, nameBUF); u8g2.drawXBMP(8, 3, 16, 16, gateBUF);}

    if (gateTempStatus[1] == 0) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameAND))) / 2)), 20, nameAND); u8g2.drawXBMP(40, 3, 16, 16, gateAND);}
    if (gateTempStatus[1] == 1) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameOR))) / 2)), 20, nameOR); u8g2.drawXBMP(40, 3, 16, 16, gateOR);}
    if (gateTempStatus[1] == 2) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 20, nameNAND); u8g2.drawXBMP(40, 3, 16, 16, gateNAND);}
    if (gateTempStatus[1] == 3) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 20, nameNOR); u8g2.drawXBMP(40, 3, 16, 16, gateNOR);}
    if (gateTempStatus[1] == 4) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 20, nameXOR); u8g2.drawXBMP(40, 3, 16, 16, gateXOR);}
    if (gateTempStatus[1] == 5) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 20, nameXNOR); u8g2.drawXBMP(40, 3, 16, 16, gateXNOR);}
    if (gateTempStatus[1] == 6) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 20, nameNOT); u8g2.drawXBMP(40, 3, 16, 16, gateNOT);}
    if (gateTempStatus[1] == 7) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 20, nameBUF); u8g2.drawXBMP(40, 3, 16, 16, gateBUF);}

    if (gateTempStatus[2] == 0) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameAND))) / 2)), 20, nameAND); u8g2.drawXBMP(72, 3, 16, 16, gateAND);}
    if (gateTempStatus[2] == 1) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameOR))) / 2)), 20, nameOR); u8g2.drawXBMP(72, 3, 16, 16, gateOR);}
    if (gateTempStatus[2] == 2) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 20, nameNAND); u8g2.drawXBMP(72, 3, 16, 16, gateNAND);}
    if (gateTempStatus[2] == 3) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 20, nameNOR); u8g2.drawXBMP(72, 3, 16, 16, gateNOR);}
    if (gateTempStatus[2] == 4) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 20, nameXOR); u8g2.drawXBMP(72, 3, 16, 16, gateXOR);}
    if (gateTempStatus[2] == 5) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 20, nameXNOR); u8g2.drawXBMP(72, 3, 16, 16, gateXNOR);}
    if (gateTempStatus[2] == 6) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 20, nameNOT); u8g2.drawXBMP(72, 3, 16, 16, gateNOT);}
    if (gateTempStatus[2] == 7) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 20, nameBUF); u8g2.drawXBMP(72, 3, 16, 16, gateBUF);}

    if (gateTempStatus[3] == 0) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameAND))) / 2)), 20, nameAND); u8g2.drawXBMP(104, 3, 16, 16, gateAND);}
    if (gateTempStatus[3] == 1) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameOR))) / 2)), 20, nameOR); u8g2.drawXBMP(104, 3, 16, 16, gateOR);}
    if (gateTempStatus[3] == 2) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 20, nameNAND); u8g2.drawXBMP(104, 3, 16, 16, gateNAND);}
    if (gateTempStatus[3] == 3) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 20, nameNOR); u8g2.drawXBMP(104, 3, 16, 16, gateNOR);}
    if (gateTempStatus[3] == 4) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 20, nameXOR); u8g2.drawXBMP(104, 3, 16, 16, gateXOR);}
    if (gateTempStatus[3] == 5) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 20, nameXNOR); u8g2.drawXBMP(104, 3, 16, 16, gateXNOR);}
    if (gateTempStatus[3] == 6) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 20, nameNOT); u8g2.drawXBMP(104, 3, 16, 16, gateNOT);}
    if (gateTempStatus[3] == 7) {u8g2.drawUTF8(((112 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 20, nameBUF); u8g2.drawXBMP(104, 3, 16, 16, gateBUF);}

    if (gateTempStatus[4] == 0) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameAND))) / 2)), 52, nameAND); u8g2.drawXBMP(8, 35, 16, 16, gateAND);}
    if (gateTempStatus[4] == 1) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameOR))) / 2)), 52, nameOR); u8g2.drawXBMP(8, 35, 16, 16, gateOR);}
    if (gateTempStatus[4] == 2) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 52, nameNAND); u8g2.drawXBMP(8, 35, 16, 16, gateNAND);}
    if (gateTempStatus[4] == 3) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 52, nameNOR); u8g2.drawXBMP(8, 35, 16, 16, gateNOR);}
    if (gateTempStatus[4] == 4) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 52, nameXOR); u8g2.drawXBMP(8, 35, 16, 16, gateXOR);}
    if (gateTempStatus[4] == 5) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 52, nameXNOR); u8g2.drawXBMP(8, 35, 16, 16, gateXNOR);}
    if (gateTempStatus[4] == 6) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 52, nameNOT); u8g2.drawXBMP(8, 35, 16, 16, gateNOT);}
    if (gateTempStatus[4] == 7) {u8g2.drawUTF8(((16 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 52, nameBUF); u8g2.drawXBMP(8, 35, 16, 16, gateBUF);}

    if (gateTempStatus[5] == 0) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameAND))) / 2)), 52, nameAND); u8g2.drawXBMP(40, 35, 16, 16, gateAND);}
    if (gateTempStatus[5] == 1) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameOR))) / 2)), 52, nameOR); u8g2.drawXBMP(40, 35, 16, 16, gateOR);}
    if (gateTempStatus[5] == 2) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 52, nameNAND); u8g2.drawXBMP(40, 35, 16, 16, gateNAND);}
    if (gateTempStatus[5] == 3) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 52, nameNOR); u8g2.drawXBMP(40, 35, 16, 16, gateNOR);}
    if (gateTempStatus[5] == 4) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 52, nameXOR); u8g2.drawXBMP(40, 35, 16, 16, gateXOR);}
    if (gateTempStatus[5] == 5) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 52, nameXNOR); u8g2.drawXBMP(40, 35, 16, 16, gateXNOR);}
    if (gateTempStatus[5] == 6) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 52, nameNOT); u8g2.drawXBMP(40, 35, 16, 16, gateNOT);}
    if (gateTempStatus[5] == 7) {u8g2.drawUTF8(((48 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 52, nameBUF); u8g2.drawXBMP(40, 35, 16, 16, gateBUF);}

    if (gateTempStatus[6] == 0) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameAND))) / 2)), 52, nameAND); u8g2.drawXBMP(72, 35, 16, 16, gateAND);}
    if (gateTempStatus[6] == 1) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameOR))) / 2)), 52, nameOR); u8g2.drawXBMP(72, 35, 16, 16, gateOR);}
    if (gateTempStatus[6] == 2) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNAND))) / 2)), 52, nameNAND); u8g2.drawXBMP(72, 35, 16, 16, gateNAND);}
    if (gateTempStatus[6] == 3) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNOR))) / 2)), 52, nameNOR); u8g2.drawXBMP(72, 35, 16, 16, gateNOR);}
    if (gateTempStatus[6] == 4) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameXOR))) / 2)), 52, nameXOR); u8g2.drawXBMP(72, 35, 16, 16, gateXOR);}
    if (gateTempStatus[6] == 5) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameXNOR))) / 2)), 52, nameXNOR); u8g2.drawXBMP(72, 35, 16, 16, gateXNOR);}
    if (gateTempStatus[6] == 6) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameNOT))) / 2)), 52, nameNOT); u8g2.drawXBMP(72, 35, 16, 16, gateNOT);}
    if (gateTempStatus[6] == 7) {u8g2.drawUTF8(((80 - ((u8g2.getUTF8Width(nameBUF))) / 2)), 52, nameBUF); u8g2.drawXBMP(72, 35, 16, 16, gateBUF);}
  }

  u8g2.drawXBMP(104, 38, 16, 19, A);

  if (menu == 0) u8g2.drawRFrame(0,0,32,32,5);
  if (menu == 1) u8g2.drawRFrame(32,0,32,32,5);
  if (menu == 2) u8g2.drawRFrame(64,0,32,32,5);
  if (menu == 3) u8g2.drawRFrame(96,0,32,32,5);
  if (menu == 4) u8g2.drawRFrame(0,32,32,32,5);
  if (menu == 5) u8g2.drawRFrame(32,32,32,32,5);
  if (menu == 6) u8g2.drawRFrame(64,32,32,32,5);

  if (menuOpen) {
    if (menu == 0) {u8g2.drawUTF8(2, 20, "<"); u8g2.drawUTF8(27, 20, ">");}
    if (menu == 1) {u8g2.drawUTF8(34, 20, "<"); u8g2.drawUTF8(59, 20, ">");}
    if (menu == 2) {u8g2.drawUTF8(66, 20, "<"); u8g2.drawUTF8(91, 20, ">");}
    if (menu == 3) {u8g2.drawUTF8(98, 20, "<"); u8g2.drawUTF8(123, 20, ">");}
    if (menu == 4) {u8g2.drawUTF8(2, 52, "<"); u8g2.drawUTF8(27, 52, ">");}
    if (menu == 5) {u8g2.drawUTF8(34, 52, "<"); u8g2.drawUTF8(59, 52, ">");}
    if (menu == 6) {u8g2.drawUTF8(66, 52, "<"); u8g2.drawUTF8(91, 52, ">");}
  }

  u8g2.setFont(u8g2_font_squeezed_r6_tr);
  if (!(menuOpen == true & menu == 0)) {
    u8g2.setCursor(3, 3); u8g2.print(digitalRead(A_IN1));
    if (gateStatus[0] != 6 & gateStatus[0] != 7) {u8g2.setCursor(3, 11); u8g2.print(digitalRead(A_IN2));}
    u8g2.setCursor(26, 7); u8g2.print(digitalRead(A_OUT));
  }

if (!(menuOpen == true & menu == 1)) {
    u8g2.setCursor(35, 3); u8g2.print(digitalRead(B_IN1));
    if (gateStatus[1] != 6 & gateStatus[1] != 7) {u8g2.setCursor(35, 11); u8g2.print(digitalRead(B_IN2));}
    u8g2.setCursor(58, 7); u8g2.print(digitalRead(B_OUT));
  }

  if (!(menuOpen == true & menu == 2)) {
    u8g2.setCursor(67, 3); u8g2.print(digitalRead(C_IN1));
    if (gateStatus[2] != 6 & gateStatus[2] != 7) {u8g2.setCursor(67, 11); u8g2.print(digitalRead(C_IN2));}
    u8g2.setCursor(90, 7); u8g2.print(digitalRead(C_OUT));
  }
  
  if (!(menuOpen == true & menu == 3)) {
    u8g2.setCursor(99, 3); u8g2.print(digitalRead(D_IN1));
    if (gateStatus[3] != 6 & gateStatus[3] != 7) {u8g2.setCursor(99, 11); u8g2.print(digitalRead(D_IN2));}
    u8g2.setCursor(122, 7); u8g2.print(digitalRead(D_OUT));
  }

 if (!(menuOpen == true & menu == 4)) {
    u8g2.setCursor(3, 35); u8g2.print(digitalRead(E_IN1));
    if (gateStatus[4] != 6 & gateStatus[4] != 7) {u8g2.setCursor(3, 43); u8g2.print(digitalRead(E_IN2));}
    u8g2.setCursor(26, 39); u8g2.print(digitalRead(E_OUT));
  }

if (!(menuOpen == true & menu == 5)) {
    u8g2.setCursor(35, 35); u8g2.print(digitalRead(F_IN1));
    if (gateStatus[5] != 6 & gateStatus[5] != 7) {u8g2.setCursor(35, 43); u8g2.print(digitalRead(F_IN2));}
    u8g2.setCursor(58, 39); u8g2.print(digitalRead(F_OUT));
  }

if (!(menuOpen == true & menu == 6)) {
    u8g2.setCursor(67, 35); u8g2.print(digitalRead(G_IN1));
    if (gateStatus[6] != 6 & gateStatus[6] != 7) {u8g2.setCursor(67, 43); u8g2.print(digitalRead(G_IN2));}
    u8g2.setCursor(90, 39); u8g2.print(digitalRead(G_OUT));
  }

  u8g2.setFont(u8g2_font_squeezed_r7_tr);
}